// ////////////////////////////////////////////////////
//
// StdView.cpp : implementation of the CStdView class
// Based on class COGLView : public CView
// Inspect COGLView for standard OpenGL initialization
//
////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Std.h"

#include "StdDoc.h"
#include "StdView.h"
#include "math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////
// You can put definitions, variables and helperfunctions
// here. Functions that does not rely on CStdView member
// variables or functions
#define PI					3.14159265359f
#define Bane_Bredde			1.0f
#define Bane_Radius			10.0f
#define Bane_Forskyvning	3.0f
#define Kule_Radius			1.0f
#define Flat_Lengde			31.0f // er lik ca en halv sirkeomkrets
#define Start_Parameter		0.5f

//////////////////////////////////////////////////////////
// CStdView

IMPLEMENT_DYNCREATE(CStdView, COGLView)

BEGIN_MESSAGE_MAP(CStdView, COGLView)
	//{{AFX_MSG_MAP(CStdView)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_COMMAND(ID_KULE_TEST, OnKuleTest)
	ON_WM_TIMER()
	ON_COMMAND(ID_RUN_SELF, OnRunSelf)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////
// CStdView construction/destruction

CStdView::CStdView()
{
	// init angles
	m_Vx=20.0f;
	m_Vy=60.0f;
	// no action
	KuleIsRunning=FALSE;
	WeAreRunning=FALSE;
	m_hRC=NULL;
	m_hDC=NULL;
};

CStdView::~CStdView()
{
}

////////////////////////////////////////////////////////////
// CStdView diagnostics
// You should usually not touch these:
#ifdef _DEBUG
void CStdView::AssertValid() const
{COGLView::AssertValid();}
void CStdView::Dump(CDumpContext& dc) const
{COGLView::Dump(dc);}

CStdDoc* CStdView::GetDocument() // non-debug version is inline
{	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CStdDoc)));
	return (CStdDoc*)m_pDocument;
}
#endif //_DEBUG

//////////////////////////////////////////////////////////////
// CStdView operations

void CStdView::DrawScene()
{
	// will need a quadric
	GLUquadricObj *glpQ;
	glpQ=gluNewQuadric();

	// Clear the color and depth buffers.
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		
	// Set up for scene
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	if(WeAreRunning)
	{
	//*********************************************
	// POSSIBLE PART OF SOLUTION DELOPPGAVE D
		// go like the sphere and look "forward"

		glRotatef(-90.0f,0.0,0.0,1.0);

		GLfloat next_kule_x=
			getX(kule_t+0.05f,Bane_Radius-Kule_Radius);
		GLfloat next_kule_y=
			getY(kule_t+0.05f,Bane_Radius-Kule_Radius);
		GLfloat next_kule_z=
			getZ(kule_t+0.05f,Bane_Radius-Kule_Radius)+Bane_Bredde/2.0f;
		
		gluLookAt(	kule_x,kule_y,kule_z,
					next_kule_x,next_kule_y,next_kule_z,
					0.0f,0.0f,1.0f);

		DrawLoop();
	// END POSSIBLE PART OF SOLUTION DELOPPGAVE D
	//*********************************************
	}
	else
	{
		// watching from distance
		gluLookAt(	0.0f,0.0f,-20.0f,
					0.0f,0.0f,0.0f,
					0.0f,1.0f,0.0f);

		// tilt it around x
		glRotatef(m_Vx,1.0f,0.0f,0.0f);
		// tilt it around y
		glRotatef(m_Vy,0.0f,1.0f,0.0f);
		

		//*********************************************
		// POSSIBLE PART OF SOLUTION DELOPPGAVE C and (B)
		// draw the thing
		glPushMatrix();
		DrawLoop();
		glPopMatrix();
		// show kule ?
		if(KuleIsRunning)
		{
			glTranslatef(kule_x,kule_y,kule_z);
			CMaterial::SetMaterial(BRONZE);
			gluSphere(glpQ,Kule_Radius,12,12);
		}
		// END POSSIBLE PART OF SOLUTION DELOPPGAVE C
		//*********************************************
	}
	glFlush();
	gluDeleteQuadric(glpQ);

}

//*****************************************
// POSSIBLE SOLUTION DELOPPGAVE A
// ref.: to drawing in assigment:
// loop has origo i center of circle
//	x twds right, y up, and z against viewer
//	A is at (Flat_Lengde,-rad,0)
//	B is at (0,-rad,0),(0,-rad,-Bane_Forskyvning)
//	C is at (-Flat_Lengde,-rad,-Bane_Forskyvning)
// Each of the function decides which curvesegment
// we are on:
//		AB, t<0
//		Circle, 0<t<1
//		BC, t>1

GLfloat CStdView::getX(GLfloat t,GLfloat rad)
{
	if(t<0.0f)
		return (GLfloat)(-fabs(t)*Flat_Lengde);
	if(t>1.0f)
		return (GLfloat)(-(1.0-t)*Flat_Lengde);
	// correct with (270/360)=0.75 to make angle start right
	return rad*(GLfloat)cos((double)2*PI*(t+0.75));
}
GLfloat CStdView::getY(GLfloat t,GLfloat rad)
{
	if(t<0.0f)
		return -rad;
	if(t>1.0f)
		return -rad;
	return rad*(GLfloat)sin((double)2*PI*(t+0.75));
}
GLfloat CStdView::getZ(GLfloat t,GLfloat rad)
{
	if(t<0.0f)
		return 0.0f;
	if(t>1.0f)
		return -Bane_Forskyvning;
	return -t*Bane_Forskyvning;
}
// END POSSIBLE SOLUTION DELOPPGAVE A
//**********************************************

//**********************************************
// POSSIBLE SOLUTION DELOPPGAVE B

void CStdView::DrawLoop()
{
	// could, and probably should, be
	// more sophisticated, but it works
	// as a minimum solution

	CMaterial::SetMaterial(RED_PLASTIC,GL_FRONT);
	CMaterial::SetMaterial(EMERALD,GL_BACK);
	glBegin(GL_TRIANGLE_STRIP);
	GLfloat t=-Start_Parameter;
	while(t<1.0+Start_Parameter)
	{
		GLfloat x=getX(t,Bane_Radius);
		GLfloat y=getY(t,Bane_Radius);
		GLfloat z=getZ(t,Bane_Radius);
		// need a normal
		if((t<0.0f)||(t>1.0f))
			glNormal3f(0.0f,1.0f,0.0f);// up
		else
			glNormal3f(-x,-y,-z);// normal twds centrum in loop	
		glVertex3f(x,y,z);
		glVertex3f(x,y,z+Bane_Bredde);
		t+=0.03f;
	}
	glEnd();
}
// END POSSIBLE SOLUTION DELOPPGAVE B
//*******************************************

void CStdView::InitializeRContext()
{
	// here we set the attributes that will be constant
	// during the windows lifetime
	// must be adjusted
    GLfloat ambient[] = { 0.4f, 0.4f, 0.4f, 1.0f };
    GLfloat diffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
    GLfloat position[] = { 200.0f, 300.0f, 100.0f, 0.0f };

    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LESS);

    glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
    glLightfv(GL_LIGHT0, GL_POSITION, position);

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,1);
	
	glEnable(GL_NORMALIZE);
	glShadeModel(GL_SMOOTH);

    glClearColor(0.8f, 0.8f, 0.8f, 0.0f);
}

void CStdView::SetPerspective(int cx, int cy)
{
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	gluPerspective(70.0f, (GLdouble)cx/cy, 0.1f, 70.0f);
	glMatrixMode(GL_MODELVIEW);
}


void CStdView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// user has pressed left button.
	// make a note of this and prepare for intuitive rotation
	m_PLast=point;
	SetCapture();
	CView::OnLButtonDown(nFlags, point);
}

void CStdView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// free capture
	ReleaseCapture();
	CView::OnLButtonUp(nFlags, point);
}

void CStdView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// if left button is down 
	if(nFlags && MK_LBUTTON)
	{	
		// and has moved a significant dist
		if((abs(m_PLast.x-point.x)>0)&&(abs(m_PLast.y-point.y)>0))
		{
			// we find an intuitive rot angle
			CRect R;
			GetClientRect(&R);
			// switch axes from win to opengl
			m_Vy+=90.0f*((GLfloat (point.x-m_PLast.x))/GLfloat (R.Width()));
			m_Vx+=90.0f*((GLfloat (point.y-m_PLast.y))/GLfloat (R.Height()));
			// and force redraw 
			m_PLast=point;
			RedrawWindow();
		}
	}
}

//***********************************************
// POSSIBLE SOLUTION OPPGAVE C

void CStdView::OnKuleTest() 
{
	// Vis fram og start kule
	KuleIsRunning=TRUE;
	SetTimer(1,100,NULL);
	kule_t=-Start_Parameter;
}

void CStdView::OnTimer(UINT nIDEvent) 
{
	// if kule is running show next pos
	KillTimer(1);
	kule_x=getX(kule_t,Bane_Radius-Kule_Radius);
	kule_y=getY(kule_t,Bane_Radius-Kule_Radius);
	kule_z=getZ(kule_t,Bane_Radius-Kule_Radius)+Bane_Bredde/2.0f;
	RedrawWindow();
	
	kule_t+=0.05f;
	if(kule_t < 1.0+Start_Parameter)
		SetTimer(1,100,NULL);
	else
	{
		WeAreRunning=FALSE;
		KuleIsRunning=FALSE;
	}
	CView::OnTimer(nIDEvent);
}

void CStdView::OnRunSelf() 
{
	// Vis fram og start en tur
	WeAreRunning=TRUE;
	SetTimer(1,100,NULL);
	kule_t=-Start_Parameter;
}

// END - POSSIBLE SOLUTION OPPGAVE C
//***********************************************

